// - zXcars: zXcar - Base Vehicle -
// -         By: -zXc-90-9-9ER    -

/*
 zXcars is a modification for Deus Ex 1 which was developed by: Ion Storm Inc. and published by: Eidos Interactive on June 26, 2000.
 The mod gives the player(s) the ability to give cars within Deus Ex.
 Copyright (C) 2012  Justin Curr.

 This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License.
    
 This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
//---------------------------------
class zXcar extends Keypad;
//-: The reason why I extended zXcar from Keypad is so I can make a hack without having to rewrite/ edit the player class - gamemode.
//-: (personally I wanted to keep this simple)

//-: Vehicle Limit/Rate Variables:
var() float MaxSpeed; //-: This is the max speed that the vehicle will move forward.
var() float Acceleration; //-: This is the rate that the vehicle will increase speed.
var() float ReverseSpeed; //-: This is the max speed the vehicle will move in reverse.
var() float BreakRate; //-: This is the rate whitch the break will slow the vehicle.

var() float WheelLimit; //-: Wheel limit is the max angle the wheels will tilt. (how sharp you can turn)
var() float WheelRate; //-: This is the rate inwhich the Wheel will tilt. (how fast for it to hit WheelLimit)
var() float FrictionMultiplier; //-: the bigger the number the faster the vehicle will slowdown on corners. (default 0.03)
var() float SpeedSlasher; //-: Use this to slash the actual velocity it uses (currentVelocity/SpeedSlasher)

var() float CoastFriction; //-: This is the friction applyed to the vehicle while it is coasting.
var() float TireGrip; //-: This is the brink (speed) when tires will slide. (drifting)
var() int	Gears; //-: This is the rate which sound pitch will change. (set Gears to 1 if you want a gokart sound, 5 for car)
var() vector SeatPosition; //-: This is the offset from the vehicle where the driver seat is.
var() float	Fuel; //-: This is the max fuel the vehicle holds.
var() float	FuelDrain; //-: This is how much fuel drains every tick (0 is inf)

var() Sound VehicleSound; //-: The sound which the vehicle makes while its turned on.
var() int VehicleMotorPitch; //-: The pitch that the motor will sit at then double for max speed.
var() int VehicleMotorVol; //-: The pitch that the motor will sit at then double for max speed.

//-: Vehicle Physics Values: (I guess I will comment these too, even knowing they shouldn't be edited.)
var	  float currentVelocity; //-: The rate which the tires are pushing forward.
var	  float currentWheelAngle; //-: The angle which the wheel is currently at.
var	  float currentGforceAngle; //-: The angle which Gforce is currently pulling.

//-: Vehicle Inputs
var	int		Input_Forward; //-: This is the input to tell the physics if player is pressing (W/, S/ or Space)
var int		Input_Right; //-: This is the input to tell the physics if player is pressing (A/ or D)
var bool	Engine; //-: This is Engine stuff so it doesn't waste gas
var float	calculatedspeed; //-: the value after speed has been slashed;
var float	ExitHeight; //-: Laggy servers people seem to die so exit height will set their hight before exiting.

var bool ExitTrigger; //-: This is used to trigger the guy out of the vehicle when the UI is closed (for fix of stuck glitch)
var vector lastLocation; //-: This is used to detect if the vehicle has hit a wall.
var vector lastVelocity; //-: Same reason as last location.

//-: The Driver of the vehicle.
var DeusExPlayer Driver;
var zXcarUI Vehicle_Interface;

//------------------------------------------------------------------------------
replication
{
  reliable if ((Role < ROLE_Authority) && (bNetOwner))
      RecievedInput; //-: So the vehicle UI can give keyinputs to vehicle.
  reliable if (bNetOwner)
	Fuel, Engine, calculatedspeed, Vehicle_Interface; //-: Give shit to the UI
}

function HackAction(Actor VehicleFrobber, bool bHacked)
{
	local DeusExPlayer Player;
	
	//-: If the player is already in the vehicle then return.
	if (Vehicle_Interface != None) return;

	//-: Convert Actor to Deus Ex Player.
	Player = DeusExPlayer(VehicleFrobber);

	//-: If Actor is actually a Player then let him in the vehicle.
	if (Player != None && Driver == None)
	{
	  Driver = Player;
	  SetOwner(Player);
	  ExitTrigger = false;
	  
	  Player.SetCollision(false, false, false);
	  Player.bCollideWorld = false;
	  Player.SetPhysics(PHYS_Flying);
	  Player.ReducedDamageType = 'All';
	  ExitHeight = Player.Location.Z - Location.Z;
	  
      Player.ActivateKeypadWindow(Self, false);
	}
}
// ----------------------------------------------------------------------
//-: Place the Player in the Vehicle and setup the UI.
// DEUS_EX AMSD Bounce back call from player so function rep works right.
// ----------------------------------------------------------------------
simulated function ActivateKeypadWindow(DeusExPlayer VehiclePlayer, bool currentAction)
{
	//-: Set some local variables.
	local DeusExRootWindow root;
	
	if(currentAction == false)
	{

		//-: Get Player window.
		root = DeusExRootWindow(VehiclePlayer.rootWindow);
		
		//-: If we got the window then add the vehicle HUD to it.
		if (root != None)
		{
			Vehicle_Interface = zXcarUI(root.InvokeUIScreen(Class'zXcarUI', True));
			//keypadwindow = Vehicle_Interface;
	      
			//-: Do not shade the screen while in vehicle.
			root.MaskBackground(False);
	      
			//-: IF the UI has been setup then run it.
			if (Vehicle_Interface != None)
			{
				//-: Set vehicle the interface owner to self.
				Vehicle_Interface.Owner_Vehicle = Self;
				//-: Set Player using the vehicle UI.
				Vehicle_Interface.player = VehiclePlayer;
				//-: Run Setup.
				Vehicle_Interface.InitData();
			}
		}
	}	
}

//=============================================================================
// -: The Vehicles Tick and Physics calculations
//=============================================================================

function Tick(float deltaTime)
{
	local rotator VehicleDirection;
	//-: Run Parent code because we are nice.
	Super.Tick(deltaTime);
	
	if(ExitTrigger && Driver != None)
	{
		Exit_Vehicle();
	}
	//-: If there is a driver place him in the correct spot.
	if(Driver != None)
	{
		Driver.SetLocation(Location+SeatPosition);
		Driver.LoopAnim('SitBreathe',0.15);
	}
	else
	{Input_Forward	= 0; Input_Forward	= 0;}

	if(SpeedSlasher != 0)calculatedspeed = (currentVelocity/SpeedSlasher);
	else calculatedspeed = (currentVelocity);
	//------------------------------------------------
	//-: The vehicles physics calcuations
	if(Physics == PHYS_Rolling)
	{
	
		if(currentVelocity > 0)
		{
			if(Location.X == lastLocation.X){currentVelocity -= abs(lastVelocity.X);}
			if(Location.Y == lastLocation.Y){currentVelocity -= abs(lastVelocity.Y);}
		}
		else if(currentVelocity < 0)
		{
			if(Location.X == lastLocation.X){currentVelocity += abs(lastVelocity.X);}
			if(Location.Y == lastLocation.Y){currentVelocity += abs(lastVelocity.Y);}
		}
		
		//-: Update the position in which the Vehicle is moving.
		Velocity.X = (currentVelocity*cos((Rotation.Yaw * (0.0000958251953125))));
		Velocity.Y = (currentVelocity*sin((Rotation.Yaw * (0.0000958251953125))));
		Velocity.Z = 0;
		
		//-: This updates the slide simulation.
		Velocity.X += (currentGforceAngle*cos(((Rotation.Yaw + 16383.999999999999999999999999996) * (0.0000958251953125))));
		Velocity.Y += (currentGforceAngle*sin(((Rotation.Yaw + 16383.999999999999999999999999996) * (0.0000958251953125))));
		Velocity.Z = 0;	
		
		//-: This sill set the slide force down since tires have friction.
		if(currentGforceAngle > 0)
		{
			if(currentGforceAngle <= TireGrip){currentGforceAngle = 0;}
			else{currentGforceAngle -= TireGrip;}
		}
		else if(currentGforceAngle < 0)
		{
			if(currentGforceAngle >= TireGrip){currentGforceAngle = 0;}
			else{currentGforceAngle += TireGrip;}
		}
		
		//-: If the wheel is not centered turn and add gforce to the vehicle.
		if(currentWheelAngle != 0)
		{
			VehicleDirection.Yaw = Rotation.Yaw + currentWheelAngle*currentVelocity;
			SetRotation(VehicleDirection);
			
			if(currentVelocity > 0)currentVelocity -= abs(currentWheelAngle*(currentVelocity*FrictionMultiplier));
			else if(currentVelocity < 0)currentVelocity += abs(currentWheelAngle*(currentVelocity*FrictionMultiplier));
		}
		
	}
	else if(VSize(Velocity) < 1)
	{
		SetPhysics(PHYS_Rolling);
	}
	
	//-: Save last location so we can check if we hit something.
	lastLocation = Location;
	lastVelocity = Velocity;
	
	//-: Make sure it doesn't run while we have no fuel
	if(Fuel <= 0){Engine = false;}
	else if(Engine){Fuel -= FuelDrain;}
		
	//-: Handling pedals.
	if((Input_Forward == 1) && (currentVelocity < MaxSpeed) && (Engine) )
	{ // This speeds things up
		currentVelocity += Acceleration;
	}
	else if((Input_Forward == 2) && (currentVelocity > -ReverseSpeed) && (Engine))
	{	//-: Slow Down Homie.
		currentVelocity -= Acceleration;
	}
	else if(Input_Forward == 3)
	{	//-: This is when using your break
		if(currentVelocity > 0)
		{	if(currentVelocity <= BreakRate){ currentVelocity = 0; }
			else
			{                             
				currentVelocity -= BreakRate;
			}
		}
		if(currentVelocity < 0)
		{	if(currentVelocity >= -CoastFriction){ currentVelocity = 0; }
			else 
			{
				currentVelocity += CoastFriction;
			}
		}
	}
	else if(Input_Forward == 0)
	{	//-: This is when its idleing
		if(currentVelocity > 0)
		{	if(currentVelocity <= CoastFriction){ currentVelocity = 0; }
			else
			{                             
				currentVelocity -= CoastFriction;
			}
		}
		if(currentVelocity < 0)
		{	if(currentVelocity >= -CoastFriction){ currentVelocity = 0; }
			else 
			{
				currentVelocity += CoastFriction;
			}
		}
	}
	//-: This sets the wheel to the correct direction.
	if((Input_Right == 1) && (currentWheelAngle > -WheelLimit))
	{	// -0- Turn Left!!!!!
		currentWheelAngle -= WheelRate;
	}
	else if((Input_Right == 2) && (currentWheelAngle < WheelLimit))
	{	// -0- Turn Right here man.
		currentWheelAngle += WheelRate;
	}
	else
	{	// -0- Basically this Sets the wheel to the center when not being pressed.
		// -0- For "noobs' that don't know how to use sticky steering wheels.
		if(currentWheelAngle > 0)
		{	if(currentWheelAngle <= WheelRate){ currentWheelAngle = 0; }
			else {currentWheelAngle -= WheelRate;}
		}
		if(currentWheelAngle < 0)
		{	if(currentWheelAngle >= -WheelRate){ currentWheelAngle = 0; }
			else {currentWheelAngle += WheelRate;}
		}
	}
	
	//------------------------------------------------
	
	//-: Gears will operate here
	if (currentVelocity > 0 && Engine)
	{
		AmbientSound = VehicleSound; // DEBUG COMMAND
		SoundPitch = VehicleMotorPitch+(VehicleMotorPitch*(currentVelocity/MaxSpeed));
		SoundVolume = VehicleMotorVol;
	}
	else if (currentVelocity < 0 && Engine)
	{
		AmbientSound = VehicleSound; // DEBUG COMMAND
		SoundPitch = VehicleMotorPitch+(VehicleMotorPitch*abs(-currentVelocity/ (-ReverseSpeed)));
		SoundVolume = VehicleMotorVol;
	}
	else if(Engine)
	{
		AmbientSound = VehicleSound;
		SoundPitch = VehicleMotorPitch;
		SoundVolume = VehicleMotorVol;
	}
	else
	{
		AmbientSound = None;
		SoundVolume = VehicleMotorVol;
		SoundPitch = VehicleMotorVol;
	}
}

//===================================================
// -0- CONTROLLER FUNCTIONS
//===================================================

function int RecievedInput(int Action)
{	
	switch(Action) 
	{	
		case -1:	Input_Forward	= 0; break;
		case -2:	Input_Forward	= 0; break;
		case -3:	Input_Right		= 0; break;
		case -4:	Input_Right		= 0; break;
		
		case -5:	Input_Forward	= 0; break;
		
		case 0:	ExitTrigger = true; break;
		
		case 1:	Input_Forward	= 1; break;
		case 2:	Input_Forward	= 2; break;
		case 3:	Input_Right		= 1; break;
		case 4:	Input_Right		= 2; break;
		
		case 5:	Input_Forward	= 3; break;
		case 6:	if(Engine){Engine = false;}else{Engine = true;} break;
	}
} // OH Shit that was easy.
function Exit_Vehicle()
{	
	local vector tempexit;
	tempexit.Z = ExitHeight;
	Driver.SetLocation(Location+tempexit);
	Driver.SetCollision(true, true, true);
	Driver.bCollideWorld = true;
	Driver.SetPhysics(PHYS_Falling);
	Driver.ReducedDamageType = '';
	Driver = None;
}
defaultproperties
{
     bCanBeBase=True
     ItemName="50cc Go-Kart"
     Mesh=LodMesh'zXcars_beta03.Gokart_M'
     SoundRadius=32
     CollisionRadius=50.000000
     CollisionHeight=15.000000
     Mass=120.000000
     Buoyancy=45.000000
     bCollideWorld=True
     Velocity=(Z=-5)
     Physics=PHYS_Falling
     bFixedRotationDir=True
     
     //Bug Fixes
     bHackable=False
     NetPriority=9.000000
     
     SeatPosition=(Z=20)
     
     MaxSpeed=600
	 Acceleration=5
	 ReverseSpeed=200
	 BreakRate=25
	 SpeedSlasher=15

     WheelLimit=0.8
	 WheelRate=0.04
	 FrictionMultiplier=0.035

     CoastFriction=2
	 TireGrip=400
	 Gears=1
	 
	 Fuel=15
	 FuelDrain=0.0001
	 
	 VehicleSound=Sound'DeusExSounds.Generic.MotorHum'
	 VehicleMotorPitch=75
	 VehicleMotorVol=240
}